#include "integrators.hpp"
#include "function.hpp"

// Gravity centre calculation for functionally defined figure.
// This is the part of NACRE project.
// NACRE -- Numerical Analysis / Computer REsearch.
//-------------------------------------------------------------------

// Copyleft (c) 2006 Alexey Beshenov <bav.272304@gmail.com>.

// NACRE is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation.

// NACRE is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details (the COPYING file).

//-----------------------------------------------------------------------------

void usage();

int main (int argc, char* argv[])
    {

    cout << "Gravity centre calculation for functionally defined figure" << endl;
    cout << "V.2006-04-17 [" << LIB_ID << "], copyleft (c) 2006 Alexey Beshenov" << endl;
    cout << COPY_NOTICE << endl << endl;

// number of arguments including keys, values, program name:
    int pn=7;

    if( argc != pn ) { usage(); exit(EXIT_FAILURE); }

//-----------------------------------------------------------------------------

    int d, f, n; long double a, b;
    bool a_def=0, b_def=0, f_def=0;

    for (d=1; d<argc; d++) {
        if (!strcmp(argv[d], "-a")) {
            if (d<argc-1) { a = strtold(argv[d+1], NULL); a_def=1; }
            }
        if (!strcmp(argv[d], "-b")) {
            if (d<argc-1) { b = strtold(argv[d+1], NULL); b_def=1; }
            }
        if (!strcmp(argv[d], "-f")) {
            if (d<argc-1) { f = d+1; f_def=1; }
            }
        }

//-----------------------------------------------------------------------------

    if( n < 1 ) {
        cerr << "ERROR! n must be an integer >= 1" << endl;
        exit(EXIT_FAILURE);
        }

    if (!a_def) { cerr << "ERROR! Missing parameter: -a" << endl; }
    if (!b_def) { cerr << "ERROR! Missing parameter: -b" << endl; }
    if (!f_def) { cerr << "ERROR! Missing parameter: -f" << endl; }
    if (!(a_def&&b_def&&f_def)) {
        usage(); exit(EXIT_FAILURE); cerr << endl;
        }

//-----------------------------------------------------------------------------

    ld_array ints; int c, zh;

    for (c=0; c<=2; c++) {
        ostringstream expression;

        if (c == 0) { expression << argv[f]; }
        if (c == 1) { expression << "(" << argv[f] << ")*x"; }
        if (c == 2) { expression << "(" << argv[f] << ")^2"; }

        char *expression_cs = new char [(int)strlen(expression.str().c_str())+1];

        for (zh=0; zh<(int)strlen(expression.str().c_str()); zh++) {
            expression_cs[zh] = expression.str().c_str()[zh];
            }
        expression_cs[zh++] = '\0';

        try { parser.Compile(addbrackets(expression_cs)); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }

        ints.push_back (PPA_calculate (a, b, EPS_LIM_T, PPA_ALG));

        delete[] expression_cs;
        }

    long double x_coord, y_coord;

    if (fabs(ints[0]) > EPS_LIM_T) {
        x_coord = ints[1]/ints[0];
        y_coord = (long double)0.5*ints[2]/ints[0];
        }

    else {
// for example, -1 1 sin(x)
        x_coord = (b+a)/(long double)2.0;
        y_coord = (long double)0.0;
        }

    cout << argv[f] << " at [" << a << "," << b << "]:" << endl;

    cout.precision(OUTPUT_EPS); cout.setf(ios_base::fixed);
    cout << "~= (" << x_coord << ", " << y_coord << ")" << endl;

    PROC_TIME

        return 0;

    }


//-----------------------------------------------------------------------------

void usage()
    {
    cout << "Usage: -a <long double> -b <long double> -f \"<string>\"" << endl << endl;
    cout << "Interval: [a, b]" << endl << endl;
    cout << builtin_functions;
    }
