#include "integrators.hpp"
#include "function.hpp"

// Polynomial interpolation and integration.
// This is the part of NACRE project.
// NACRE -- Numerical Analysis / Computer REsearch.
//-------------------------------------------------------------------

// Copyleft (c) 2006 Alexey Beshenov <bav.272304@gmail.com>.

// NACRE is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation.

// NACRE is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details (the COPYING file).

//-----------------------------------------------------------------------------

void usage();

int main (int argc, char* argv[])
    {

    cout << "Polynomial interpolation and integration" << endl;
    cout << "V.2006-04-22 [" << LIB_ID << "], copyleft (c) 2006 Alexey Beshenov" << endl;
    cout << COPY_NOTICE << endl << endl;

// number of arguments including keys, values, program name:
    int pn=7;

    if( argc < pn ) { usage(); exit(EXIT_FAILURE); }

//-----------------------------------------------------------------------------

    int d; long double a, b;
    bool a_def=0, b_def=0;

    for (d=1; d<argc; d++) {
        if (!strcmp(argv[d], "-a")) {
            if (d<argc-1) { a = strtold(argv[d+1], NULL); a_def=1; }
            }
        if (!strcmp(argv[d], "-b")) {
            if (d<argc-1) { b = strtold(argv[d+1], NULL); b_def=1; }
            }
        }

//-----------------------------------------------------------------------------

    if (!a_def) { cerr << "ERROR! Missing parameter: -a" << endl; }
    if (!b_def) { cerr << "ERROR! Missing parameter: -b" << endl; }
    if((argc-1-a_def-b_def)%2 != 0) { cerr << "ERROR! You must specify pairs of x_i y_i" << endl; }
    if (!(a_def&&b_def)||((argc-1-a_def-b_def)%2 != 0)) {
        usage(); exit(EXIT_FAILURE); cerr << endl;
        }

//-----------------------------------------------------------------------------

    int n = (argc-5)/2;                           // Number of points
// 1 -- program name, 4 -- keys

    ld_array x; ld_array y;
// points

    int z, i;
    for (z=2,i=0; z<argc && i<n; z++) {
// if x and y are not key identifiers or key values:
        if ( strcmp(argv[z-1], "-a") && strcmp(argv[z-1], "-b") &&
            strcmp(argv[z], "-a") && strcmp(argv[z], "-b") &&
        strcmp(argv[z+1], "-a") && strcmp(argv[z+1], "-b") ) {
            x.push_back (strtold(argv[z++],NULL));
            y.push_back (strtold(argv[z],NULL));

            cout << "   | x_" << (long double)i << "=" << x[x.size()-1] << ", y_" << (long double)i++ << "=" << y[y.size()-1] << ";" << endl;
            }
        }

    cout << endl;

    cout.precision(OUTPUT_EPS); cout.setf(ios_base::fixed);

    cout << "Interpolated polynomial: " << interpolate(x,y) << endl << endl;

    cout.unsetf(ios_base::fixed);

    try { parser.Compile(addbrackets(interpolate(x,y))); }
    catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }

    long double x_min, x_max;
    x_min = x[0], x_max = x[0];
    for (z=0;z<n;z++) {
        if (x_min > x[z]) { x_min = x[z]; }
        if (x_max < x[z]) { x_max = x[z]; }
        }

    if ( (a<=b && (a<x_min||b>x_max)) || (a>=b && (b<x_min||a>x_max)) ) {
        cerr << "WARNING! Integration beyond given points!" << endl;
        cerr << "Define points at " << "[" << a << "," << b << "]" << endl << endl;
        }

    long double eps = EPS_LIM_T;
    cout << "\\int_{" << a << "}^{" << b << "} [I_n(x)]dx" << endl;

    cout.setf(ios_base::fixed);
    cout << PPA_calculate (a, b, eps, PPA_ALG) << endl;

    PROC_TIME

        return 0;

    }


//-----------------------------------------------------------------------------

void usage()
    {
    cout << "Usage: -a <long double> -b <long double>   x_1 y_1 ... x_n y_n" << endl;
    }
