#include "integrators.hpp"
#include "function.hpp"

// Integral calculation with Prefered Precision Algorithm.
// This is the part of NACRE project.
// NACRE -- Numerical Analysis / Computer REsearch.
//-------------------------------------------------------------------

// Copyleft (c) 2006 Alexey Beshenov <bav.272304@gmail.com>.

// NACRE is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation.

// NACRE is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details (the COPYING file).

//-------------------------------------------------------------------

void usage();

int main (int argc, char* argv[])
    {

    cout << "Integral calculation with Prefered Precision Algorithm" << endl;
    cout << "V.2006-04-17 [" << LIB_ID << "], copyleft (c) 2006 Alexey Beshenov" << endl;
    cout << COPY_NOTICE << endl << endl;

// number of arguments including keys, values, program name:
    int pn=9;

    if( argc != pn && argc != pn+2 ) { usage(); exit(EXIT_FAILURE); }

//-----------------------------------------------------------------------------

    int d, f, g; long double a, b, eps;
    bool a_def=0, b_def=0, f_def=0, g_def=0, eps_def=0;

    for (d=1; d<argc; d++) {
        if (!strcmp(argv[d], "-a")) {
            if (d<argc-1) { a = strtold(argv[d+1], NULL); a_def=1; }
            }
        if (!strcmp(argv[d], "-b")) {
            if (d<argc-1) { b = strtold(argv[d+1], NULL); b_def=1; }
            }
        if (!strcmp(argv[d], "-f")) {
            if (d<argc-1) { f = d+1; f_def=1; }
            }
        if (!strcmp(argv[d], "-g")) {
            if (d<argc-1) { g = d+1; g_def=1; }
            }
        if (!strcmp(argv[d], "-eps")) {
            if (d<argc-1) { eps = (long double)strtold(argv[d+1], NULL); eps_def=1; }
            }
        }

//-----------------------------------------------------------------------------

    if( LIMIT_EPS_T && eps < EPS_LIM_T ) {
        cout.setf(ios_base::scientific);
        cout << "ERROR! Can't work with precisions < " << EPS_LIM_T << endl;
        cout << "Using " << EPS_LIM_T << endl << endl;
        cout.unsetf(ios_base::scientific);
        eps = EPS_LIM_T;
        }

    if( LIMIT_EPS_B && eps > EPS_LIM_B ) {
        cout.setf(ios_base::scientific);
        cout << "ERROR! Can't work with precisions > " << EPS_LIM_B << endl;
        cout << "Using " << EPS_LIM_B << endl << endl;
        cout.unsetf(ios_base::scientific);
        eps = EPS_LIM_B;
        }

    if (!a_def) { cerr << "ERROR! Missing parameter: -a" << endl; }
    if (!b_def) { cerr << "ERROR! Missing parameter: -b" << endl; }
    if (!f_def) { cerr << "ERROR! Missing parameter: -f" << endl; }
    if (!eps_def) { cerr << "ERROR! Missing parameter: -eps" << endl; }
    if (!(a_def&&b_def&&f_def&&eps_def)) {
        usage(); exit(EXIT_FAILURE); cerr << endl;
        }

//-------------------------------------------------------------------

    if (!g_def) {                                 // function of real variable
        try { parser.Compile(addbrackets(argv[f])); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }
        cout << "\\int_{" << a << "}^{" << b << "} [" << argv[f] << "]dx" << endl;
        cout.precision(OUTPUT_EPS); cout.setf(ios_base::fixed);
        cout << "~= " << PPA_calculate(a,b,eps,PPA_ALG) << endl;
        }

    else {                                        // function of complex variable
        long double Re, Im;
        try { parser.Compile(addbrackets(argv[f])); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }
        Re = PPA_calculate(a,b,eps,PPA_ALG);
        try { parser.Compile(addbrackets(argv[g])); }
        catch(TError error) { puts(error.error); exit(EXIT_FAILURE); }
        Im = PPA_calculate(a,b,eps,PPA_ALG);
        cout << "\\int_{" << a << "}^{" << b << "} [(" << argv[f] << ") + i*(" << argv[g] << ")]dx" << endl;
        ostringstream z;
        z.precision(OUTPUT_EPS); z.setf(ios_base::fixed);
        z << Re;
        if (fabs(Im) > EPS_LIM_T) { z << " + i*" << Im; }
        cout << "~= " << z.str() << endl;
        }

    PROC_TIME

        return 0;

    }


//-----------------------------------------------------------------------------

void usage()
    {
    cout << "Usage: -a <long double> -b <long double> -f \"<string>\" [-g \"<string>\"] -eps <long double>" << endl << endl;
    cout << "[z(x)]dx, z(x) = f(x) + i*g(x) if -g defined (function of complex variable)" << endl;
    cout << "[f(x)]dx if -g not defined (function of real variable)" << endl;
    cout << "Interval: [a, b]" << endl << endl;
    cout.setf(ios_base::scientific);
    cout << "Precision: eps (" << EPS_LIM_B << " -- " << EPS_LIM_T << ")" << endl;
    cout.unsetf(ios_base::scientific);
    cout << builtin_functions;
    }
